# $Id: customerConfig.py 3695 2010-06-04 10:55:35Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.1.0/src/customerConfig.py $
import os
#from ConfigParser import ConfigParser
import ConfigParser
from os.path import isfile
import sys
import traceback
import re
from sets import Set
from glob import glob
from sys import stderr
import pprint
from string import lower

class CustomerConfig:
    """
      Provide simple configuration file support, for multiple languages (default: 'en' i.e. English).
      The code uses the standard python configparser:  http://docs.python.org/library/configparser.html
      The format of the config file is thus prescribed by the configparser.

      The most important call is:

      get(option, section, default)

      The value returned is:
        1 The value actually found. If not present:
        2 The value of the 'default' parameter. If not present:
        3 The content of the 'option' parameter. If not present:
      X 4 None, raise CustomerConfigException
    """
    def __init__(self, infra = None):
        self.pp = pprint.PrettyPrinter(indent=2)
        self.infra = infra
        self.logger     = infra.logging.getLogger('meresco.configs')
        self.sections = {}
        self.configFileParserDict = {};
        self.configFiles = []

        self.wildcardList = infra.baseParser.get('bzv', 'configFiles')
        globlist = self.wildcardList.split(',')
        self.logger.warning( self.wildcardList + " globlist: " + ','.join(globlist))
        
        for el in globlist:
            fileList  = glob(el)
            if len(fileList) == 0:
                
                self.logger.error( 'No config files found for :"' + el + "'") 
                continue
              
            for file in fileList:
                self.configFileParserDict[file] = None
                self.configFiles.append(file)
                
        self.readConfigFiles()
          
    def readConfigFiles(self):
        self.sections = {}
        for file in self.configFiles:
            configParser = ConfigParser.ConfigParser()
            try:
                configParser.read(file)
            except Exception, e:
                self.logger.error('Error: Could not parse configfile: "' + file + '" ' + traceback.format_exc())
                continue
            
            self.configFileParserDict[file] = configParser 
                            
            for section in configParser.sections():
                if section in self.sections:
                    self.logger.error('DUPLICATED SECTION: [' + section + ']')
                    self.logger.error('Skipped  in:  "' + file + '"') 
                    self.logger.error('Also found in:"' + self.sections[section]['filename'] + '"')
                    continue
                
                dict = {}
                for option in configParser.options(section):
                        dict[option] = configParser.get(section, option)

                self.sections[section] = {}
                self.sections[section]['dict'] = dict
                self.sections[section]['parser'] = configParser
                self.sections[section]['filename'] = file
#                self.sections[section]['dict'] = ConfigSection(configParser, section, self.infra)
            
    def getSectionFullname(self, section):
        return self.sections[section]['filename'] + '[' + section + ']'

    def crlfList(self, string, skipEmptyLines = True):
        collection = []
        if string != None:
              collection =  string.split("\n")
              if skipEmptyLines:
                    collection = filter(lambda x: len(x), collection)
        return collection

    # returns a list of strings by splitting the variable's "\n" and removing empty lines
    def getValueAsList(self, option, default = [], section = None, skipEmptyLines = True, language = "en"):
            string = self.get( option, section=section, default=default, language=language )
            self.logger.debug("language: " + language +" list: " + string)
            return self.crlfList(string, skipEmptyLines)

    def getSectionDictList(self,  section = None, skipEmptyLines = True, language = "en"):
        hash = {}
        sectionParser = self.getSectionParser(section)
        options  = sectionParser.items(section)
        for option,value in options:
            # ignore value
            hash[option] = self.crlfList(self._get(option, parser=sectionParser, section=section, language=language), skipEmptyLines)
        return hash

    def getSectionNormalized(self, section = None, language="en" ):
        hash = {}
        sectionParser = self.getSectionParser(section)
        options  = sectionParser.items(section)
        for option,value in options:
            # ignore value
            valueList = self.crlfList(self._get(option, parser=sectionParser, section=section, language=language))
            for valueElement in valueList:
                hash[valueElement] = option
        return hash

    def normalizeOption(self,option):
        langexpr = re.compile('\_\w{2}$')

        res = langexpr.search(option)
        if res != None:
            pos = res.start()
            normalized = option[:pos]
        else:
            normalized = option
        return normalized


    # returns a hash of an entire section, original version
    def getSectionDict(self,  section = None, skipEmptyLines = True, language = "en", mergeDict = {}):
        sectionParser = self.getSectionParser(section)
        options  = sectionParser.items(section)
        dict = {}
        if mergeDict != {}:
                dict = mergeDict
                
        for option,value in options:
            normalized = self.normalizeOption(option)
            if normalized in mergeDict:
                self.logger.warning('getSectionDict: Merge overwrites data. Key: "' +  normalized + '" ' + self.infra.prpt('Value', dict[normalized] ) +
                                    self.infra.prpt('with new value',value))
                self.logger.error('Called at: ' + self._callstack())

 
            dict[normalized] = self._get(normalized, parser=sectionParser, section=section, language=language )

        return dict

    def getSectionTupleList(self,  section = None, skipEmptyLines = True, language = "en"):
        sectionParser = self.getSectionParser(section)
        dict = self.getSectionDict(section, skipEmptyLines, language)
        list = []
        for option in dict:
            value = dict [ option ]
            list.append((option,value))
        return list
 
    # another way to present the hash, doesn't work yet, needs an update() method,  FIXME
    def xgetSectionDict(self, section = None):
        return self.sections[section]['dict']
        
    def getSectionParser(self,section):
        if (section == None or section == '') :
            self.logger.error("No section specified")
            sane = False
             
        elif section in self.sections:
            parser = self.sections[section]['parser']
        else:
            self.logger.error('Section not found:[' + section + '] in filelist: \n    ' + '\n    '.join( self.configFileParserDict ))
            self.logger.error('Called at: ' + self._caller(3))
            parser = None
        return parser

    def _callstack(self):
        stack = traceback.format_stack()
        #el = stack.pop()
        return ''.join(stack)

    def _caller(self, level):
        stack = traceback.format_stack()
        while level:
            level -= 1
            stack.pop()
        el = stack.pop()
        return el

    def _get(self,option=None, parser=None, section=None, language='en'):
        # nb, the passed option must be a /normalized/ i.e. should not contain any _xx extensions
        self.logger.debug('language: "' + language + "'" )
        value = None

        if option !=  self.normalizeOption(option):
            self.logger.error('_get: option contains illegal extension: "_xx": "' + option +'"')
            raise IOError

        reqoption = option
        if language != 'en': reqoption = option + "_" + language 
        
        try:
            self.logger.debug('_get trying ' + self.getSectionFullname(section) + reqoption) 
            value = parser.get(section, reqoption)
        except ConfigParser.NoOptionError:
            try:
                if language != 'en': 
                    reqoption = option
                else:
                    reqoption = option + language
                self.logger.info('default option failed, trying language variation: ' + self.getSectionFullname(section) + reqoption)
                self.logger.info(self._caller (3)) 
                value = parser.get(section, reqoption)
            except:
                self.logger.warning ('"' + str(option) + '[' + language + ']'  +
                        '" NOT specified ' + 'in file: "' +
                        self.sections[section]['filename'] + '[' + str(section) + ']')
                
                self.logger.warning('Called at: ' + self._caller(3))
                self.logger.warning(self._caller (3)) 
        
        self.logger.debug('returning: "' + str(value) + '" for caller: ' + self._caller(3))
        return value

    def get(self, option = None, default = None, section = None, language='en'):
        sane = True
        value = None

        if (option == None or option == ''):
            self.logger.error("No valid option specified")
            sane = False

        if (sane):
            parser = self.getSectionParser(section)
            if parser is None: sane = False 
        
        if (sane):
            value = self._get( option=option, section=section, parser=parser, language=language)
        else:
            if (value == ''):
                value = "*Incomplete Request*"
            self.logger.error("CustomerConfig: Incomplete Request.")
            self.logger.error('Called at: ' + self._caller(2))

        if value is None:
            if default is not None: value = default
            elif option is not None: value = option
            else: value = '*UNDEFINED*'
            self.logger.warn('No correct value found, returning: "' + value +'"')
            self.logger.warn('Called at: ' + self._caller(2))

        return value


# another way to present the hash, doesn't work yet, needs an update() method,  FIXME
class ConfigSection(object):
    def __init__(self, config, section, infra):
        self.logger = infra.logger
        self._config = config
        self._section = section
        
    def __getitem__(self, name):
        self.logger.debug('name: "' +  str(name) + '"')
        res = ''
        try:
            res = self._config.get(self._section, lower(str(name)))
        except:
            pass
        return res

    def __contains__(self, name):
        return self._config.get(self._section, lower(str(name)))
