# $Id: infra.py 3576 2010-05-17 13:57:24Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.1.0/src/infra.py $
import os
import logging
import logging.config
import pprint
import sys
from ConfigParser import ConfigParser
from traceback import format_tb
from bzv.customerConfig import CustomerConfig
import syslog
#
# Create a hive mind:
# http://www.eecho.info/Echo/python/singleton/
#

class Borg(object):
    _state = {}
    def __new__(cls, *args, **kw):
        ob = super(Borg, cls).__new__(cls, *args, **kw)
        ob.__dict__ = cls._state
        return ob

class Infra (Borg):

    def mayday(self, message):
        sys.stderr.write (message)
        syslog.syslog(message)


    def __init__(self, startupMessage = '', bzvConfigFile = None ):
        self.pp = pprint.PrettyPrinter(indent=2)

        # bootstrap configuration
        self.baseParser = ConfigParser()

        if not os.path.exists(bzvConfigFile):
            message = 'File not found: "' + bzvConfigFile + '"'
            self.mayday(message)
            raise IOError
        
        try:
            self.baseParser.read(bzvConfigFile)
            self.loggingConfigfile = self.baseParser.get('bzv','loggingConfiguration')
            self.configpath = self.baseParser.get('bzv','configpath')

        except Exception, e:
            message = 'ERROR: Could not read basic configuration from: "' + bzvConfigFile + '" ' 
            message += ''.join(e) + "\n"
            self.mayday(message)
            raise 
                        
        # initialise logging
        self.startLogging(startupMessage)

        # initialise configuration
        self.configRoot = CustomerConfig(self)


    def startLogging(self,startupMessage):
        # import logging
        self.logging=logging
        try:
            logging.config.fileConfig(self.loggingConfigfile)

        except Exception, e:
            message = 'ERROR: Could not parse loggingConfigfile: "' + self.loggingConfigfile + '" '
            message += ''.join(e) + "\n"
            self.mayday(message)

        self.logger     = self.logging.getLogger('meresco.infra')
        logging.info(startupMessage)
        self.logger.debug(startupMessage)

    def prpt(self, label, param):
        return label + "\n" + self.pp.pformat(param)

    def dd(self, label, param):
        self.logging.debug(self.prpt(label, param))

    def getLoggingInstance(self):
        return self.logging
